"""Django application configuration for fluid_design.
"""
from django.apps import AppConfig

class FluidDesignConfig(AppConfig):
    """Django application configuration for fluid_design.
    """
    name = 'fluid_design'
